/* 
   |---------------------------------------------------------------------------------------------------------------------
   | Program   : I2CAddress.ino  
   | Version   : 1.03
   | Author    : Mike G7GTN - South Devon
   | Email     : mikeg7gtn@gmail.com
   | Date      : April 2015
   | IDE       : V1.0.5
   |---------------------------------------------------------------------------------------------------------------------
   |
   | Description :
   |
   |              List addresses of devices connected to the I2C Bus.
   |              User output is to a normal PAL or NTSC composite video signal.
   |
   | I/O         :
   |
   |              Analog  Pin A4   = I2C SDA 
   |              Analog  Pin A5   = I2C SCL 
   |              Digital Pin D4   = Switch Connections uses internal Pullup resistor
   |              Digital Pin D5   = Switch Clear TV Screen uses internal Pullup resistor
   |              Digital Pin D6   = Switch Scan Bus uses internal Pullup resistor
   |              Digital Pin D9   = Sync via 1K Resistor
   |              Digital Pin D7   = Video via 470R Resistor 
   |
   | Libraries   :
   |
   |              TVOut http://code.google.com/p/arduino-tvout/downloads/detail?name=TVout_R5.91.zip&can=2&q=
   |
   |              The above Library needs to be correctly installed in your Ardunio/Libraries folder to compile
   |              this project.
   |
   | V1.03       : Added push button debounce routine.
   |
   |---------------------------------------------------------------------------------------------------------------------

*/

const char Version [] = "1.03 ";
const char Author [] = " G7GTN ";
const int Connections = 4;                           // Connections
const int Clearscreen = 5;                           // Clear screen button on Digital I/O Pin 5 no resistor
const int Startscan = 6;                             // Scan bus button on Digital I/O Pin 6 no resistor
const int LedPin = 13;                               // Ardunio D13 LED
const int debounceDelay = 35;                        // milliseconds to wait until stable
byte count = 0;
 
#include <TVout.h>
#include <fontALL.h>
#include "bitmaps.h"
#include "schematic.h"
#include <Wire.h>
const uint8_t WIDTH = 120;                           // Set screen resolution 
const uint8_t HEIGHT = 96; 

TVout TV;

void projecttitle()
{
 TV.clear_screen(); 
 TV.bitmap(4,19,TVOlogo); 
 TV.println("  ");
 TV.println(" I2C Device Address Lister.");
 TV.print(73,73," V");                              // Across,down,<text>
 TV.println(Version); 
 TV.print(73,80,Author);
 TV.println(3,91,"* Press start to scan bus *");
}  

// if scan & clear buttons both held down
void instructions()
{
   TV.clear_screen();
   TV.bitmap(8,1,schematic);
   TV.println(14,9,"I2C DEVICE CONNECTIONS");
   TV.println(10,87,"* Press start to scan *");   
}  
  
// Pin Digital I/O 6 Switch
void start()
{
  scan();  
}

// Push buttons & I/O state setup
void setupiodirection()
{
   pinMode(LedPin, OUTPUT); 
   pinMode(Clearscreen, INPUT);
   digitalWrite(Clearscreen, HIGH);                // Use internal pull-up resistors   
   pinMode(Startscan, INPUT);
   digitalWrite(Startscan, HIGH);
   pinMode(Connections, INPUT);
   digitalWrite(Connections, HIGH); 
   digitalWrite(LedPin, LOW);                      // Turn off digital D13 LED   
}  

// Scan I2C Bus
void scan()
{
   TV.clear_screen();                              // Clear screen first
   for (byte i = 1; i < 127; i++)	           // Range of addresses
   {
    Wire.beginTransmission (i);
    if (Wire.endTransmission () == 0)
      {
      TV.print ("Found address: ");
      TV.print (i, DEC);
      TV.print (" (0x");
      TV.print (i, HEX);
      TV.println (")");
      count++;
      delay (1);                          
      }                                   
  }                                                 
  TV.println("  ");
  TV.println ("Scan Finished.");
  TV.println("  ");
  TV.print ("Found ");
  TV.print (count, DEC);
  TV.println (" I2C device(s).");
  digitalWrite(LedPin, LOW);                        // Turn off D13
  count = 0;                                        // Reset address counter
}  

// Following code from Ardunio Cookbook http://www.amazon.co.uk/Arduino-Cookbook-Michael-Margolis/dp/0596802471 
// Page 142
boolean debounce(int pin)
{
boolean state;
boolean previousState;
previousState = digitalRead(pin);                   // store switch state
for(int counter=0; counter < debounceDelay; counter++)
{
delay(1);                                           // wait for 1 millisecond
state = digitalRead(pin);                           // read the pin
if( state != previousState)
{
counter = 0;                                       // reset the counter if the state changes
previousState = state;                             // and save the current state
}
}
// here when the switch state has been stable longer than the debounce period
if(state == LOW)                                   // LOW means pressed (because pull-ups used)
return true;
else
return false;
}

void setup() 
{
 setupiodirection(); 
 Wire.begin(); 
 TV.begin(PAL,WIDTH, HEIGHT);                      // TV.begin(NTSC,WIDTH, HEIGHT)
 TV.select_font(font4x6);                          // Use smallest font
 projecttitle();
}

void loop() 
{
 if(debounce(Startscan))
 {
     digitalWrite(LedPin, HIGH);
     start();
 }
 if(debounce(Clearscreen))
 {
     projecttitle();
 } 
 if(debounce(Connections))
 {
     instructions();
 }   
}
